"""
Default settings you can import your settings.py
"""
import os

# Define available webfonts
ICOMOON_WEBFONTS = {
    #'Default': {
        #'fontdir_path': 'static/fonts', # Where the font files belong and where they will be deployed
        #'csspart_path': 'static/css/icomoon_icons.scss', # Where the css part with all icon selectors will be written
    #},
}

# Private mode require to be logged to reach the gallery, use False to grant access 
# to anonymous users
ICOMOON_PRIVATE = True

# Manifest filename, this is a Icomoon standard so you should not change it
ICOMOON_MANIFEST_FILENAME = "selection.json"

# Path to the CSS template that will contain icon map, this relative to templates directory
ICOMOON_CSS_TEMPLATE = "icomoon/icon_map.css"

# Structure and file requirements for the Icomoon webfont archive
# Also paths are relative within ZIP archive
ICOMOON_ARCHIVE_REQUIREMENT = {
    # Path where belong the font files
    'font_dir': 'fonts',
    # Allowed font format extensions
    'extensions': ['ttf', 'svg', 'eot', 'woff', 'woff2'],
}



# -*- coding: utf-8 -*-
"""
App views
"""
import json, os

#from django.db import models
from django.conf import settings
from django.views import generic

from .parser import WebfontStore


class WebfontIconListFreeView(generic.TemplateView):
    """
    View listing icon getted from Icomoon manifest
    """
    template_name = "icomoon/index.html"
    
    def get_context_data(self, **kwargs):
        context = super(WebfontIconListFreeView, self).get_context_data(**kwargs)
        
        webfonts = getattr(settings, 'ICOMOON_WEBFONTS', {})
        if webfonts is not None:
            webfont_store = WebfontStore(settings.ICOMOON_MANIFEST_FILENAME)
            webfont_store.fetch(webfonts)
            context['webfont_store'] = webfont_store
            
        return context

if getattr(settings, 'ICOMOON_PRIVATE', True):
    from braces.views import LoginRequiredMixin
    
    class WebfontIconListView(LoginRequiredMixin, WebfontIconListFreeView):
        pass
else:
    class WebfontIconListView(WebfontIconListFreeView):
        pass



"""
Icomoon manifest parser
"""
import json, os
from collections import OrderedDict
from operator import itemgetter

from icomoon.utils import IcomoonSettingsError, extend_webfont_settings

class WebfontStore(object):
    """
    Webfont store to collect every given manifest
    """
    def __init__(self, manifest_filename):
        self.manifest_filename = manifest_filename
        self.manifests = OrderedDict()
        self.errors = {}
            
    def fetch(self, webfonts):
        """
        Open all defined manifest files and parse them
        
        @webfonts: Dict of Dict from settings.ICOMOON_WEBFONTS
        """
        sorted_keys = sorted(webfonts.keys())
        for webfont_name in sorted_keys:
            self.get(webfont_name, webfonts[webfont_name])
            
    def get(self, webfont_name, webfont_settings):
        """
        Get a manifest file and parse it
        
        @webfont_name: Webfont key name
        @webfont_settings: Dict of webfont settings
        """
        try:
            webfont_settings = extend_webfont_settings(webfont_settings)
        except IcomoonSettingsError as e:
            self.errors[webfont_name] = "Invalid webfont settings for '{}': {}".format(webfont_name, e.value)
            return
            
        filepath = os.path.join(webfont_settings['fontdir_path'], self.manifest_filename)
        if os.path.exists(filepath):
            self.manifests[webfont_name] = self.parse_manifest(filepath)
        else:
            self.errors[webfont_name] = "Filepath for webfont <strong>{name}</strong> does not exists: <code>{filepath}</code>".format(name=webfont_name, filepath=filepath)
            
    def get_icon_key(self, elem):
        """
        Return the icon name to be used as a sorting key
        """
        return elem.get('properties').get('name')

    def parse_manifest(self, fp):
        """
        Open the JSON file then find the icons and some options
        
        @fp is either a file object or a string containing the file path to open
        """
        # Given a string for file path to open
        if isinstance(fp, basestring):
            fp = open(fp, 'rb')
        
        with fp as json_file:
            webfont_manifest = json.load(json_file)
        
        # Get the font set prefix to know the css classname
        icon_prefix = webfont_manifest.get('preferences').get('fontPref').get('prefix')
        
        # Get sorted icons
        icons_map = OrderedDict()
        sorted_entries = sorted(webfont_manifest.get('icons'), key=self.get_icon_key)
        for icon_entry in sorted_entries:
            name = icon_entry.get('properties').get('name')
            code = icon_entry.get('properties').get('code')
            hexa_code = hex(code)

            icons_map[name] = {
                'class_name': icon_prefix + name,
                'int': code,
                'hex': hexa_code,
                'unicode': 'U+'+''.join(hex(code).split('x')[1:]).upper(),
                'utf8': '\\'+''.join(hex(code).split('x')[1:]).lower(),
            }
        
        return icons_map
            
    def get_manifests(self):
        return self.manifests



class IcomoonSettingsError(Exception):
    """Exception to raise when the settings are invalid"""
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

def extend_webfont_settings(webfont_settings):
    """
    Valid webfont settings, fill missing optional key then return them
    """
    if 'fontdir_path' not in webfont_settings:
        raise IcomoonSettingsError("Webfont settings miss the required key item 'fontdir_path'")
    if 'csspart_path' not in webfont_settings:
        webfont_settings['csspart_path'] = None

    return webfont_settings



# -*- coding: utf-8 -*-
"""
App URLs
"""
from django.conf.urls import *

from .views import WebfontIconListView

urlpatterns = patterns('',
    url(r'^$', WebfontIconListView.as_view(), name="index"),
)
